# NOTE: Derived from blib/lib/Getopt/Long.pm.
# Changes made here will be lost when autosplit again.
# See AutoSplit.pm.
package Getopt::Long;

#line 200 "blib/lib/Getopt/Long.pm (autosplit into blib/lib/auto/Getopt/Long/define.al)"
sub define {

    my $self = shift(@_);
    Croak (__PACKAGE__."::define requires an object as its first argument\n")
      unless defined $self && index(ref($self), '=');

    my @speclist = @_;	# local copy of the option descriptions

    local $error = $self->{error} = '';
    my $autoabbrev            = $self->{autoabbrev};
    my $ignorecase            = $self->{ignorecase};
    my $bundling              = $self->{bundling};
    my $debug                 = $self->{debug};
    my $linkage               = $self->{_linkage};
    my $userlinkage           = $self->{_userlinkage};
    my $pkg                   = $self->{_package};
    my $prefix                = $self->{_genprefix};
    my $useprefix             = $self->{_useprefix};

    print STDERR (__PACKAGE__." $Getopt::Long::VERSION ",
		  'define $Revision: 2.24 $ ',
		  "called from package \"$pkg\".",
		  "\n  ",
		  "Spec: (@speclist)",
		  "\n  ",
		  "autoabbrev=$autoabbrev,",
		  "bundling=$bundling,",
		  "getopt_compat=", $self->{getopt_compat}, ",",
		  "order=", $self->{order}, ",",
		  "\n  ",
		  "ignorecase=$ignorecase,",
		  "passthrough=", $self->{passthrough}, ",",
		  "useprefix=$useprefix,",
		  "prefix=\"$prefix\",",
		  "terminator=\"", $self->{_terminator}, "\".",
		  "\n")
	if $debug;

    unless ( defined $self->{_state_} ) {
	$self->{_state_} = [ {}, # table of arg.specs (long and abbrevs)
			     {}, # table of arg.specs (bundles)
			     {}, # alias table
			   ];
    }
    my ($opctl, $bopctl, $aliases) = @{$self->{_state_}};

    # Verify correctness of speclist.
LOOP:
    while ( @speclist > 0 ) {
	my $opt = shift (@speclist);
	my $oref;
	$oref = shift (@speclist) if @speclist && ref($speclist[0]);

	# Filter out the specials.
	if ( $opt eq '<>' ) {
	    if ( (defined $userlinkage)
		&& !defined $oref
		&& (exists $userlinkage->{$opt})
		&& ref($userlinkage->{$opt}) ) {
		$oref = $userlinkage->{$opt};
	    }
	    unless ( defined $oref && ref($oref) eq 'CODE' ) {
		$error .= "Option spec <> requires a reference to a subroutine\n";
		next;
	    }
	    $linkage->{'<>'} = $oref;
	    next;
	}

	# Empty arg means "-" option.
	$opt = "-" if $opt eq "";

	# Break the option arg into option specs and control string.
	my $o = $opt;
	my $c = ['~', '$'];
	if ( $opt =~ m{^
                      (.+)		# 1 option name and aliases
		      ([=:])		# 2 takes required/optional value
		      ([binfs])		# 3 option type
		      ([\$\@%])?	# 4 optional destination type
		      (\(.*\))?		# 5 optional value list
		      (\{.*\})?		# 6 optional repetition
		      $}x ) {
	    $o = $1;
	    my $mmin = $2 eq '=' ? 1 : 0;
	    my $type  = $3;
	    my $dst   = $4;
	    my $vlist = $5;
	    my $rept  = $6;

	    # Range specified?
	    if ( defined $rept ) {
		# Must have array destination.
		if ( defined $dst && $dst ne '@' ) {
		    $error .= "Option \"$opt\": ".
		      "repetition requires the destination to be \@";
		    next LOOP;
		}
		unless ( $rept =~ /^\{(\d+)?(,(\d?)?)?\}$/ ) {
		    $error .= "Option \"$opt\": ".
		      "illegal repetition \"$rept\"\n";
		    next LOOP;
		}
		my ($min, $max) = ($1, $3);
		$min = $mmin unless defined $min;
		# {x} -> {x,x}
		$max = $min unless defined $2;
		# min <= max?
		if (defined $max && $max < $min ) {
		    $error .= "Option \"$opt\": ".
		      "error in repetition \"$rept\" -- ".
		      "max must be >= min\n";
		    next LOOP;
		}
		$c = [$type, '@', $min, $max];
	    }
	    else {
		$c = [$type, $dst || '$', $mmin, 1];
	    }

	    # Value list specified?
	    if ( defined $vlist ) {
		if ( $vlist =~ /^\((-?\d+)\.\.(-?\d+)\)$/) {
		    if ( $type ne 'i' && $type ne 'n' ) {
			$error .= "Option \"$opt\": ".
			  "range spec requires ".
			  "the type to be \"i\" or \"n\"\n";
			next LOOP;
		    }
		    if ( $1 > $2 ) {
			$error .= "Option \"$opt\": ".
			  "range spec error -- min > max\n";
			next LOOP;
		    }
		    $type = $c->[0] = 'r'; # range
		    push (@$c, [$1, $2]);
		}
		else {
		    my @list = split (' ', substr($vlist,1,length($vlist)-2));
		    if ( $^W && @list <= 1 ) {
			warn ("Option \"$opt\": ".
			      "very few values in value list $vlist\n");
		    }
		    push (@$c, [@list]);
		}
	    }
	}
	elsif ( $opt =~ /^(.+)([!+])$/ ) {
	    $o = $1;
	    $c = [$2, '$'];
	}
	# Other errors will be caught later.

	# Handle alias names
	my @o =  split (/\|/, $o);
	my $linko;
	my $a;

	foreach $o ( @o ) {
	    my $starter;

	    if ( $o eq '-' ) {
		# empty -> '-' option
		$o = '';
	    }
	    else {
		# First, strip leading prefix so people can specify "--foo=i".
		($starter,$o) = ($1,$+)
		  if $prefix ne '()' && $o =~ /^$prefix+(.*)$/s;
		if ( $useprefix && !defined $starter ) {
		    $error .= "Missing option prefix in spec: \"$o\"\n";
		    next LOOP;
		}
		# Then check for valid form.
		if ( $o !~ /^(\?|\w+[-\w]*)$/ ||
		     ($o eq '?' && !defined $linko)) {
		    $error .= "Error in option spec: \"$o\"";
		    $error .= " of \"$opt\"" unless $o eq $opt;
		    $error .= "\n";
		    next LOOP;
		}
	    }
	    $starter = '' unless ($useprefix && defined $starter);

	    # The first of the list of aliases is the real name.
	    unless ( defined $linko ) {
		$linko = $starter.$o;
		# Force an alias if the option name is not locase.
		$a = $o unless $o eq lc($o);
		$o = lc ($o)
		  if $ignorecase > 1 
		    || ($ignorecase
			&& ($bundling ? length($o) > 1  : 1));
	    }

	    if ( $bundling && length($o) == 1 ) {
		$o = lc ($o) if $ignorecase > 1;
		if ( $c->[0] eq '!' ) {
		    $opctl->{$starter."no$o"} = [ @$c ];
		    warn ("Ignoring '!' modifier for short option $o\n");
		    $c->[0] = '~';
		}
		$opctl->{$starter.$o} = $bopctl->{$starter.$o} = $c;
	    }
	    else {
		$o = lc ($o) if $ignorecase;
		if ( $c->[0] eq '!' ) {
		    $opctl->{$starter."no$o"} = [ @$c ];
		    $c->[0] = '~';
		}
		$opctl->{$starter.$o} = $c;
	    }
	    if ( defined $a ) {
		# Note alias.
		$aliases->{$o} = $a;
	    }
	    else {
		# Set primary name.
		$a = $o;
	    }
	}
	$o = $linko;

	# If no linkage is supplied in the @speclist, copy it from
	# the userlinkage if available.
	if ( defined $userlinkage && !defined $oref ) {
	    if ( exists $userlinkage->{$o} && ref($userlinkage->{$o}) ) {
		print STDERR ("=> found userlinkage for \"$o\": ",
			      "$userlinkage->{$o}\n")
		  if $debug;
		$oref = $userlinkage->{$o};
	    }
	    else {
		# Do nothing. Being undefined will be handled later.
		next;
	    }
	}
	    
	# Copy the linkage. If omitted, link to global variable.
	if ( defined $oref ) {
	    print STDERR ("=> link \"$o\" to $oref\n")
	      if $debug;
	    if ( ref($oref) =~ /^(SCALAR|CODE)$/ ) {
		$linkage->{$o} = $oref;
	    }
	    elsif ( ref($oref) =~ /^(ARRAY)$/ ) {
		$linkage->{$o} = $oref;
		$opctl->{$o}->[1] = $bopctl->{$o}->[1] = '@';
	    }
	    elsif ( ref($oref) =~ /^(HASH)$/ ) {
		$linkage->{$o} = $oref;
		$opctl->{$o}->[1] = $bopctl->{$o}->[1] = '%';
	    }
	    else {
		$error .= "Option \"$opt\": invalid linkage\n";
	    }
	}
	else {
	    if ( 0 ) {
		# Prepare to link to global $opt_XXX variable.
		$linkage->{$o} = undef; # exists, but not defined
	    }
	    else {
		no strict 'refs';
		my $ov = $o;
		$ov = $+ 
		  if $useprefix && $prefix ne '()' && $ov =~ /^$prefix+(.*)/s;
		$ov =~ s/\W/_/g;
		my $dsttype;
		$dsttype = '$' unless defined ($dsttype = $c->[1]);
		print STDERR ("=> link \"$o\" to $dsttype$pkg",
			      "::opt_$ov\n") if $debug;
		if ( $dsttype eq '@' ) {
		    $linkage->{$o} = \@{"$pkg"."::opt_$ov"};
		}
		elsif ( $dsttype eq '%' ) {
		    $linkage->{$o} = \%{"$pkg"."::opt_$ov"};
		}
		else {
		    $linkage->{$o} = \${"$pkg"."::opt_$ov"};
		}
	    }
	}
    }

    # Bail out if errors found.
    die ($error) if $error;

    # Show the options tables if debugging.
    if ( $debug ) {
	my ($arrow, $k, $v);
	$arrow = "=> ";
	while ( ($k,$v) = each(%$opctl) ) {
	    print STDERR ($arrow, "\$opctl->{\"$k\"} = ", opctl ($v), "\n");
	    $arrow = "   ";
	}
	$arrow = "=> ";
	while ( ($k,$v) = each(%$bopctl) ) {
	    print STDERR ($arrow, "\$bopctl->{\"$k\"} = ", opctl ($v),  "\n");
	    $arrow = "   ";
	}
	$arrow = "=> ";
	while ( ($k,$v) = each(%$aliases) ) {
	    print STDERR ($arrow, "\$aliases->{\"$k\"} = \"$v\"\n");
	    $arrow = "   ";
	}
    }

    # Clear list of names, if any.
    $self->{_opnames} = undef;

    # Return self so calls can be chained.
    $self;
}

# end of Getopt::Long::define
1;
