# NOTE: Derived from blib/lib/Getopt/Long.pm.
# Changes made here will be lost when autosplit again.
# See AutoSplit.pm.
package Getopt::Long;

#line 693 "blib/lib/Getopt/Long.pm (autosplit into blib/lib/auto/Getopt/Long/findoption.al)"
# Option lookup.
sub findoption ($$$) {

    # returns (1, $opt, $arg, $opctl, $key) if okay,
    # returns (-1) if the option is not valid
    # returns (0) otherwise.

    my ($self, $opt, $argv) = @_;
    my $prefix                = $self->{_genprefix};
    my $useprefix             = $self->{_useprefix};
    my $autoabbrev            = $self->{autoabbrev};
    my $getopt_compat         = $self->{getopt_compat};
    my $ignorecase            = $self->{ignorecase};
    my $bundling              = $self->{bundling};
    my $passthrough           = $self->{passthrough};
    my $debug                 = $self->{debug};
    my $terminator            = $self->{_terminator};

    my $key;			# hash key for a hash option
    my $arg;

    print STDERR ("=> find \"$opt\", prefix=\"$prefix\"\n") if $debug;

    return (0) unless $prefix eq '()' || $opt =~ /^$prefix(.*)$/s;
    my $starter = '';
    unless ( $useprefix ) {
	$opt = $+;
	$starter = $1;
	print STDERR ("=> split \"$starter\"+\"$opt\"\n") if $debug;
    }

    my ($opctl, $bopctl, $aliases) = @{$self->{_state_}};
    my $optarg = undef;	# value supplied with --opt=value
    my $rest = undef;	# remainder from unbundling

    # If it is a long option, it may include the value.
    if (((length($starter) > 1) || ($getopt_compat && !$bundling))
	&& $opt =~ /^([^=]+)=(.*)$/s ) {
	$opt = $1;
	$optarg = $2;
	print STDERR ("=> option \"", $opt, 
		      "\", optarg = \"$optarg\"\n") if $debug;
    }

    #### Look it up ###

    my $tryopt = $opt;		# option to try
    my $optbl = $opctl;		# table to look it up (long names)
    my $c;			# points to opctl info
    my $type;			# option type
    my $dsttype = '';		# destination type
    my $argreq;			# option requires an argument 
    my $incr = 0;		# increment, if type eq '+'
    my $min;			# min #args, if repeat specified
    my $max;			# max #args, if repeat specified

    if ( $bundling && length($starter) == 1 ) {
	# Unbundle single letter option.
	$rest = substr ($tryopt, 1);
	$tryopt = substr ($tryopt, 0, 1);
	$tryopt = lc ($tryopt) if $ignorecase > 1;
	print STDERR ("=> $starter$tryopt unbundled from ",
		      "$starter$tryopt$rest\n") if $debug;
	$rest = undef unless $rest ne '';
	$optbl = $bopctl;	# look it up in the short names table

	# If bundling == 2, long options can override bundles.
	if ( $bundling == 2 and defined $rest and
	     defined ($c = $opctl->{$tryopt.$rest}) ) {
	    print STDERR ("=> $starter$tryopt rebundled to ",
			  "$starter$tryopt$rest\n") if $debug;
	    $tryopt .= $rest;
	    undef $rest;
	}
    } 

    # Try auto-abbreviation. No use to try ''.
    elsif ( $autoabbrev && $opt ne '' ) {
	# Downcase if allowed.
	$tryopt = $opt = lc ($opt) if $ignorecase;
	# Turn option name into pattern.
	my $pat = quotemeta ($opt);
	# Look up in option names.
	my @hits = grep (/^$pat/, @{$self->{_opnames}});
	print STDERR ("=> ", scalar(@hits), " hits (@hits) with \"$pat\" ",
		      "out of ", scalar(@{$self->{_opnames}}), "\n") if $debug;

	# Check for ambiguous results.
	unless ( (@hits <= 1) || (grep ($_ eq $opt, @hits) == 1) ) {
	    # See if all matches are for the same option.
	    my %hit;
	    foreach ( @hits ) {
		$_ = $aliases->{$_} if defined $aliases->{$_};
		$hit{$_} = 1;
	    }
	    # Now see if it really is ambiguous.
	    unless ( keys(%hit) == 1 ) {
		return (0) if $passthrough;
		warn ("Option \"$opt\": ambiguous (",
		      join(", ", @hits), ")\n");
		$error++;
		return (-1);
	    }
	    @hits = keys(%hit);
	}

	# Complete the option name, if appropriate.
	if ( @hits == 1 && $hits[0] ne $opt ) {
	    $tryopt = $hits[0];
	    $tryopt = lc ($tryopt) if $ignorecase;
	    print STDERR ("=> option \"$opt\" -> \"$tryopt\"\n")
		if $debug;
	}
    }

    # Map to all lowercase if ignoring case.
    elsif ( $ignorecase ) {
	$tryopt = lc ($opt);
    }

    # Check validity by fetching the info.
    $c = $optbl->{$tryopt} unless defined $c;
    unless  ( defined $c ) {
	return (0) if $passthrough;
	warn ("Option \"$opt\": unknown\n");
	$error++;
	return (-1);
    }

    # Apparently valid.
    $opt = $tryopt;

    if ( $debug ) {
	print STDERR ("=> found ", opctl($c), " for ", $opt, "\n");
    }
    ($type, $dsttype, $min, $max) = @$c;

    # If we already have a value, repeat must be 0, 1 or undefined.
    if ( (defined $optarg || defined $rest) && defined $min && $min > 1 ) {
	return (0) if $passthrough;
	my $msg = "Option \"$opt\": ";
	if ( !defined $max ) {
	    $msg .= "at least $min argument";
	    $msg .= "s" unless $min == 1;
	}
	elsif ( $min == $max ) {
	    $msg .= $min == 1 ? "an argument" : "$min arguments";
	}
	elsif ( !defined $min ) {
	    $msg .= "at most $max argument";
	    $msg .= "s" unless $max == 1;
	}
	else {
	    $msg .= "between $min and $max arguments";
	}
	$msg .= " required\n";
	warn ($msg);
	$error++;
	return (-1);
    }

    #### Determine argument status ####

    # If it is an option w/o argument, we're almost finished with it.
    if ( $type eq '~' || $type eq '!' || $type eq '+' ) {
	if ( defined $optarg ) {
	    return (0) if $passthrough;
	    warn ("Option \"$opt\": no argument allowed\n");
	    $error++;
	    unshift (@$argv, $starter.$rest) if defined $rest;
	    return (-1);
	}
	elsif ( $type eq '~' || $type eq '+' ) {
	    $arg = 1;		# supply explicit value
	    $incr = $type eq '+';
	}
	else {
	    substr ($opt, 0, 2) = ''; # strip NO prefix
	    $arg = 0;		# supply explicit value
	}
	unshift (@$argv, $starter.$rest) if defined $rest;
	return (1, $opt, $arg, $c, $key);
    }

    # Check if there is an option argument available.
    if ( defined $optarg ? ($optarg eq '') 
	 : !(defined $rest || @$argv > 0) ) {
	# Complain if this option needs an argument.
	if ( $min > 0 ) {
	    return (0) if $passthrough;
	    warn ("Option \"$opt\": argument required\n");
	    $error++;
	    return (-1);
	}
	else {
	    $arg = $type eq "s" ? '' : 0;
	}
	return (1, $opt, $arg, $c, $key);
    }
    
    # Get (possibly optional) argument.
    $arg = (defined $rest ? $rest
	    : (defined $optarg ? $optarg : shift (@$argv)));

    # Get key if this is a "name=value" pair for a hash option.
    $key = undef;
    if ( $dsttype eq '%' && defined $arg ) {
	($key, $arg) = ($arg =~ /^(.*)=(.*)$/s) ? ($1, $2) : ($arg, 1);
    }

    #### Check if the argument is valid for this option ####

    my ($ok, $val, $extra, $msg) =
      $self->valid ($c, $arg, 
		    (defined $optarg || defined $rest) ? '()' : $prefix);

    print STDERR ("=> valid ('",$c->[0],"','$arg',$min) -> ",
		  "($ok,'$val','$extra')\n") if $debug;
    if ( !$ok ) {
	if ( $min > 0 ) {
	    if ( $passthrough ) {
		unshift (@$argv, defined $rest ? $starter.$rest : $arg)
		  unless defined $optarg;
		return (0);
	    }
	    warn ("Option \"$opt\": invalid value \"$arg\"$msg\n");
	    $error++;
	    unshift (@$argv, $starter.$rest) if defined $rest;
	    return (-1);
	}
	else {
	    unshift (@$argv, defined $rest ? $starter.$rest : $arg)
	      if defined $rest || defined $arg;
	}
	return (1, $opt, $val, $c, $key);
    }
    elsif ( $arg eq $terminator && !defined $optarg && !defined $rest &&
	    !($type eq 's' && $min > 0)
	  ) {
	print STDERR ("=> reject '$arg'\n") if $debug;
	unshift (@$argv, $arg);
	$val = '';
	return (1, $opt, $val, $c, $key);
    }
    elsif ( defined $optarg && $extra ne '' ) {
	if ( $passthrough ) {
	    return (0);
	}
	warn ("Option \"$opt\": invalid value \"$arg\"$msg\n");
	$error++;
	return (-1);
    }
    else {
	if ( $bundling && $extra ne '' ) {
	    unshift (@$argv, $starter.$extra);
	}
	return (1, $opt, $val, $c, $key);
    }
}

# end of Getopt::Long::findoption
1;
