# NOTE: Derived from blib/lib/Getopt/Long.pm.
# Changes made here will be lost when autosplit again.
# See AutoSplit.pm.
package Getopt::Long;

#line 531 "blib/lib/Getopt/Long.pm (autosplit into blib/lib/auto/Getopt/Long/parse.al)"
sub parse {

    my ($self, $argv) = @_;
    Croak (__PACKAGE__."::parse requires an object as its first argument\n")
      unless defined $self && index(ref($self), '=');

    local $error = $self->{error} = 0;

    my $order                 = $self->{order};
    my $terminator            = $self->{_terminator};
    my $debug                 = $self->{debug};
    my ($opctl, $bopctl, $aliases) = @{$self->{_state_}};

    $argv = \@main::ARGV unless defined $argv;

    print STDERR (__PACKAGE__." $Getopt::Long::VERSION ",
		  'parse $Revision: 2.24 $ ',
		  "called from package \"", $self->{_package}, "\".",
		  "\n  ",
		  "ARGV: (@$argv)",
		  "\n  ",
		  "autoabbrev=", $self->{autoabbrev}, ",".
		  "bundling=", $self->{bundling}, ",",
		  "getopt_compat=", $self->{getopt_compat}, ",",
		  "order=", $self->{order}, ",",
		  "\n  ",
		  "ignorecase=", $self->{ignorecase}, ",",
		  "passthrough=", $self->{passthrough}, ",",
		  "useprefix=", $self->{_useprefix}, ",",
		  "prefix=\"", $self->{_genprefix}, "\",",
		  "terminator=\"", $self->{_terminator}, "\".",
		  "\n")
	if $debug;

    # Sort the possible long option names.
    $self->{_opnames} = [ sort(keys (%$opctl)) ] 
      if $self->{autoabbrev}  && !defined $self->{_opnames};

    my @ret = ();		# accum for non-options
    my $opt;			# current option
    my $optctl;			# control info of option
    my $argcnt = 0;		# nr. arguments assigned
    my $min = 0;		# min. arguments required
    my $max;			# max. arguments allowed

    # Process argument list
    while ( @$argv > 0 ) {

	#### Get next argument ####

	my $tryopt = shift (@$argv);
	print STDERR ("=> option \"", $tryopt, "\"\n") if $debug;

	#### Determine what we have ####

	# Consume regardless.
	if ( $argcnt > 0 && 
	     defined $optctl && $optctl->[0] eq 's' && 
	     $min > $argcnt &&
	     !defined $optctl->[4]
	   ) {
	    $self->deposit ($opt, $tryopt, '@', '', '');
	    $argcnt++;
	    next;
	}

	# Check for option list terminator.
	if ( $tryopt eq $terminator ) {

	    if ( $argcnt < $min ) {
		$error++;
		warn ("Option \"$opt\": ".
		      ((defined $max && $max > $min) ? "at least " : "").
		      "$min arguments required, only $argcnt provided\n");
	    }

	    # Finish. Push back accumulated arguments and return.
	    last;
	}

	# Consume conditionally.
	if ( $argcnt > 0 && ((defined $max) ? ($argcnt < $max) : ($argcnt < $min)) ) {

	    my ($ok, $val, $extra, $msg) =
	      $self->valid ($optctl, $tryopt);
	    print STDERR ("=> valid ('",$optctl->[0],"','$tryopt',$min) -> ",
			  "($ok,'$val','$extra')\n") if $debug;
	    if ( $ok && $extra eq '' ) {
		$self->deposit ($opt, $val, '@', '', '');
		$argcnt++;
		next;
	    }
	    if ( $argcnt < $min ) {
		$error++;
		warn ("Option \"$opt\": ".
		      ((defined $max && $max > $min) ? "at least " : "").
		      "$min arguments required, only $argcnt provided\n");
	    }
	}

	$argcnt = $min = 0;

	my $result;		# success status
	my $key;		# key (if hash type)
	my $arg;		# option argument

	($result, $opt, $arg, $optctl, $key) = 
	  $self->findoption ($tryopt, $argv);

	if ( $result > 0 ) {
	    my $type;
	    my $dsttype;
	    ($type, $dsttype, $min, $max) = @$optctl;
	    my $incr = $type eq '+';

	    $opt = $aliases->{$opt} if defined $aliases->{$opt};
	    $self->deposit ($opt, $arg, $dsttype, $incr, $key);
	    $argcnt = 1;
	    $min = $max = 1 unless defined $min;
	}

	# Ignore unknown options here. 
	elsif ( $result < 0 ) {
	}

	# Not an option. Save it if we $PERMUTE and don't have a <>.
	elsif ( $order == $PERMUTE ) {
	    # Try non-options call-back.
	    my $cb;
	    if ( (defined ($cb = $self->{_linkage}->{'<>'})) ) {
		$cb->($tryopt);
	    }
	    else {
		print STDERR ("=> saving \"$tryopt\" ",
			      "(not an option, may permute) ",
			      "ret = (@ret)\n") if $debug;
		push (@ret, $tryopt);
	    }
	    next;
	}

	# ...otherwise, terminate.
	else {
	    # Push this one back and exit.
	    unshift (@$argv, $tryopt);
	    last;
	}

    }

    # Finish.
    if ( $order == $PERMUTE && @ret > 0 ) {
	#  Push back accumulated arguments
	print STDERR ("=> restoring \"", join('" "', @ret), "\"\n")
	    if $debug;
	unshift (@$argv, @ret);
    }

    return ($error == 0);
}

# end of Getopt::Long::parse
1;
